Package: clusterExperiment
Title: Compare Clusterings for Single-Cell Sequencing
Version: 2.32.0
Description: Provides functionality for running and comparing many
    different clusterings of single-cell sequencing data or other large mRNA Expression data sets.
Authors@R: c(person("Elizabeth", "Purdom", email = "epurdom@stat.berkeley.edu",
            role = c("aut", "cre", "cph")),
	     person("Davide","Risso", role = "aut",email = "risso.davide@gmail.com"))
BugReports: https://github.com/epurdom/clusterExperiment/issues
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 3.6.0), SingleCellExperiment, SummarizedExperiment (>=
        1.15.4), BiocGenerics
Imports: methods, NMF, RColorBrewer, ape (>= 5.0), cluster, stats,
        limma, locfdr, matrixStats, graphics, parallel, BiocSingular,
        kernlab, stringr, S4Vectors, grDevices, DelayedArray (>=
        0.7.48), HDF5Array (>= 1.7.10), Matrix, Rcpp, edgeR, scales,
        zinbwave, phylobase, pracma, mbkmeans
Suggests: BiocStyle, knitr, testthat, MAST, Rtsne, scran, igraph,
        rmarkdown
VignetteBuilder: knitr
LazyData: false
LazyLoad: false
RoxygenNote: 7.3.3
biocViews: Clustering, RNASeq, Sequencing, Software, SingleCell
Collate: 'AllChecks.R' 'AllClassesCE.R' 'AllClassesCF.R'
        'AllGenerics.R' 'AllHelper.R' 'AllHelperClusterFunction.R'
        'AllHelperDendro.R' 'AllHelperFilter.R' 'JiashinJiCode.R'
        'RcppExports.R' 'addClusterings.R' 'assignUnassigned.R'
        'subsampleClustering.R' 'internalDendroFunctions.R'
        'internalClusterFunctions.R' 'internalFunctions.R'
        'builtInClusterFunctions.R' 'clusterContrasts.R'
        'clusterMany.R' 'clusterSingle.R' 'dataCreation.R'
        'deprecateFunctions.R' 'getClusterIndex.R' 'getFeatures.R'
        'getParams.R' 'getReducedData.R' 'mainClustering.R'
        'makeBlankData.R' 'makeConsensus.R' 'makeDendrogram.R'
        'makeFilterStats.R' 'makeReducedDims.R' 'mergeClusters.R'
        'plotBarplot.R' 'plotClusters.R' 'plotClustersTable.R'
        'plotClustersWorkflow.R' 'plotContrastHeatmap.R'
        'plotDendrogram.R' 'plotFeatureBoxplot.R'
        'plotFeatureScatter.R' 'plotHeatmap.R' 'plotReduceDim.R'
        'plottingHelpers.R' 'rsec.R' 'seqCluster.R' 'subsampleLoop.R'
        'subset.R' 'transformFunction.R' 'updateObject.R'
        'workflowClusters.R'
LinkingTo: Rcpp
git_url: https://git.bioconductor.org/packages/clusterExperiment
git_branch: RELEASE_3_23
git_last_commit: dae5064
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:44:47 UTC; biocbuild
Author: Elizabeth Purdom [aut, cre, cph],
  Davide Risso [aut]
Maintainer: Elizabeth Purdom <epurdom@stat.berkeley.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 04:37:28 UTC; unix
