# Generated by roxygen2: do not edit by hand

export(ClusterExperiment)
export(ClusterFunction)
export(bigPalette)
export(internalFunctionCheck)
export(listBuiltInFilterStats)
export(listBuiltInFunctions)
export(listBuiltInReducedDims)
export(listBuiltInType01)
export(listBuiltInTypeK)
export(makeBlankData)
export(makeRsecFluidigmObject)
export(massivePalette)
export(numericalAsCharacter)
export(seqCluster)
export(seqPal1)
export(seqPal2)
export(seqPal3)
export(seqPal4)
export(seqPal5)
export(setBreaks)
export(showHeatmapPalettes)
export(showPalette)
exportClasses(ClusterExperiment)
exportClasses(ClusterFunction)
exportMethods("[")
exportMethods("clusterLabels<-")
exportMethods("clusterLegend<-")
exportMethods("clusterTypes<-")
exportMethods("coClustering<-")
exportMethods("nodeLabels<-")
exportMethods("orderSamples<-")
exportMethods("primaryClusterIndex<-")
exportMethods("transformation<-")
exportMethods(ClusterExperiment)
exportMethods(ClusterFunction)
exportMethods(RSEC)
exportMethods(addClusterings)
exportMethods(addToColData)
exportMethods(algorithmType)
exportMethods(assignUnassigned)
exportMethods(checkDendrogram)
exportMethods(clusterContrasts)
exportMethods(clusterDendrogram)
exportMethods(clusterLabels)
exportMethods(clusterLegend)
exportMethods(clusterMany)
exportMethods(clusterMatrix)
exportMethods(clusterMatrixColors)
exportMethods(clusterMatrixNamed)
exportMethods(clusterSingle)
exportMethods(clusterTypes)
exportMethods(clusteringInfo)
exportMethods(coClustering)
exportMethods(colDataClusters)
exportMethods(combineMany)
exportMethods(convertClusterLegend)
exportMethods(convertToDendrogram)
exportMethods(defaultNDims)
exportMethods(dendroClusterIndex)
exportMethods(eraseMergeInfo)
exportMethods(filterData)
exportMethods(filterNames)
exportMethods(getBestFeatures)
exportMethods(getBuiltInFunction)
exportMethods(getClusterIndex)
exportMethods(getClusterManyParams)
exportMethods(getMergeCorrespond)
exportMethods(getPostProcessingArgs)
exportMethods(getReducedData)
exportMethods(getSingleClusterIndex)
exportMethods(inputType)
exportMethods(mainClustering)
exportMethods(makeConsensus)
exportMethods(makeDendrogram)
exportMethods(makeFilterStats)
exportMethods(makeReducedDims)
exportMethods(mergeClusterIndex)
exportMethods(mergeClusters)
exportMethods(mergeCutoff)
exportMethods(mergeMethod)
exportMethods(nClusterings)
exportMethods(nClusters)
exportMethods(nFeatures)
exportMethods(nInternalNodes)
exportMethods(nNodes)
exportMethods(nSamples)
exportMethods(nTips)
exportMethods(nodeIds)
exportMethods(nodeLabels)
exportMethods(nodeMergeInfo)
exportMethods(orderSamples)
exportMethods(plotBarplot)
exportMethods(plotClusterLegend)
exportMethods(plotClusters)
exportMethods(plotClustersTable)
exportMethods(plotClustersWorkflow)
exportMethods(plotCoClustering)
exportMethods(plotContrastHeatmap)
exportMethods(plotDendrogram)
exportMethods(plotFeatureBoxplot)
exportMethods(plotFeatureScatter)
exportMethods(plotHeatmap)
exportMethods(plotReducedDims)
exportMethods(primaryCluster)
exportMethods(primaryClusterIndex)
exportMethods(primaryClusterLabel)
exportMethods(primaryClusterNamed)
exportMethods(primaryClusterType)
exportMethods(recolorClusters)
exportMethods(removeClusterings)
exportMethods(removeClusters)
exportMethods(removeUnassigned)
exportMethods(renameClusters)
exportMethods(requiredArgs)
exportMethods(sampleDendrogram)
exportMethods(setToCurrent)
exportMethods(setToFinal)
exportMethods(show)
exportMethods(subsampleClustering)
exportMethods(subsetByCluster)
exportMethods(tableClusters)
exportMethods(transformData)
exportMethods(transformation)
exportMethods(updateObject)
exportMethods(workflowClusterDetails)
exportMethods(workflowClusterTable)
exportMethods(workflowClusters)
import(Matrix)
import(SingleCellExperiment)
import(SummarizedExperiment)
import(limma)
import(methods)
import(phylobase, except = plot)
importClassesFrom(DelayedArray,DelayedArray)
importClassesFrom(DelayedArray,DelayedArray1)
importClassesFrom(DelayedArray,DelayedMatrix)
importClassesFrom(DelayedArray,RleArray)
importClassesFrom(DelayedArray,RleMatrix)
importClassesFrom(HDF5Array,HDF5Matrix)
importClassesFrom(phylobase,phylo4)
importClassesFrom(phylobase,phylo4d)
importFrom(BiocGenerics,updateObject)
importFrom(BiocSingular,runPCA)
importFrom(NMF,aheatmap)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(ape,as.hclust.phylo)
importFrom(ape,as.phylo)
importFrom(ape,as.phylo.hclust)
importFrom(ape,is.ultrametric)
importFrom(ape,node.depth.edgelength)
importFrom(ape,node_depth)
importFrom(ape,node_depth_edgelength)
importFrom(ape,node_height)
importFrom(ape,node_height_clado)
importFrom(ape,nodelabels)
importFrom(ape,phydataplot)
importFrom(ape,plot.phylo)
importFrom(ape,unrooted.xy)
importFrom(cluster,daisy)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,gray)
importFrom(grDevices,rgb)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,plot)
importFrom(kernlab,specc)
importFrom(limma,makeContrasts)
importFrom(locfdr,locfdr)
importFrom(matrixStats,rowIQRs)
importFrom(matrixStats,rowMads)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowVars)
importFrom(parallel,mclapply)
importFrom(scales,alpha)
importFrom(stats,IQR)
importFrom(stats,as.dendrogram)
importFrom(stats,as.hclust)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,quantile)
importFrom(stringr,str_order)
importFrom(stringr,str_pad)
importFrom(stringr,str_sort)
importFrom(zinbwave,glmWeightedF)
importMethodsFrom(phylobase,"nodeLabels<-")
importMethodsFrom(phylobase,nNodes)
importMethodsFrom(phylobase,nTips)
importMethodsFrom(phylobase,nodeLabels)
useDynLib(clusterExperiment, .registration = TRUE)
