# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Search pairs of samples that co-cluster across subsamples
#'
#' Assume that our input is a matrix, with N columns and B rows
#' (the number of subsamples), storing integers -- the cluster labels.
#'
#' @param clusterings a matrix with the cluster labels
#'
#' @return A matrix with the co-clusters, but only the lower triangle
#' is populated.
search_pairs <- function(clusterings) {
    .Call(`_clusterExperiment_search_pairs`, clusterings)
}

makeHashKey <- function(i, j, delim) {
    .Call(`_clusterExperiment_makeHashKey`, i, j, delim)
}

splitHashKey <- function(s, delim) {
    .Call(`_clusterExperiment_splitHashKey`, s, delim)
}

subsampleLoop <- function(clList, N) {
    .Call(`_clusterExperiment_subsampleLoop`, clList, N)
}

